/*===============================================================================================
 Harmonic sounds demo!
 Tom Stratford 2007.
 
 This code demonstrates my exploration into 3D sounds and virtual voices fmod.
 The different audio tracks should combine to make one harmony.
 This demo was created using sections of code from the main lightGrow program. Any comments or refernces
 to code omitted here should be present in there.
 In the creation of the code I refered to the fmod documentation as well as the fmod examples '3d',
 'effects' and 'virtual voices'. These can be found in the fmod download package.
===============================================================================================*/

#include <SDL/SDL.h>

#include <GL/glut.h>
#include <gl/gl.h>
#include <gl/glext.h>

#include<svl/SVL.h>
#include <svl/SVLgl.h>

#include <iostream> 
#include <math.h>
#include <vector>

#include <fmod.h>
#include <fmod_dsp.h>
#include <fmod_errors.h>

#define PI 3.141592653589793
#define NUMCLIPS 17
#define NUMSOUNDS 50

// Initialise FMOD ---------------------------------------------------------

    FMOD_SYSTEM   *fmodSystem = 0;
    FMOD_SOUND    *sound3d1 = 0, *sound3d2 = 0, *sound3d3 = 0, *sound3d4 = 0, *sound3d5 = 0, *sound3d6 = 0,
                  *sound3d7 = 0, *sound3d8 = 0, *sound3d9 = 0, *sound3d10 = 0, *sound3d11 = 0, *sound3d12 = 0,
                  *sound3d13 = 0, *sound3d14 = 0, *sound3d15 = 0, *sound3d16 = 0, *sound3d17 = 0;                  
    FMOD_SOUND    *sounds[17] = {sound3d1,sound3d2,sound3d3,sound3d4,sound3d5,sound3d6,sound3d7,sound3d8,sound3d9,sound3d10,
                                sound3d11,sound3d12,sound3d13,sound3d14,sound3d15,sound3d16,sound3d17}; 
    FMOD_CHANNEL  *channel3d1 = 0, *channel3d2 = 0, *channel3d3 = 0, *channel3d4 = 0, *channel3d5 = 0, *channel3d6 = 0,
                  *channel3d7 = 0, *channel3d8 = 0, *channel3d9 = 0, *channel3d10 = 0, *channel3d11 = 0, *channel3d12 = 0,
                  *channel3d13 = 0, *channel3d14 = 0, *channel3d15 = 0, *channel3d16 = 0, *channel3d17 = 0;
    FMOD_CHANNEL  *channels[17] = {channel3d1,channel3d2,channel3d3,channel3d4,channel3d5,channel3d6,channel3d7,channel3d8,
                                   channel3d9,channel3d10,channel3d11,channel3d12,channel3d13,channel3d14,channel3d15,channel3d16,
                                   channel3d17};
    FMOD_RESULT   result;
    FMOD_DSP      *dsplowpass = 0;

// FMOD Errorcheck -------------------------------------------------------------
void ERRCHECK(FMOD_RESULT result)
{
     if (result != FMOD_OK)
     {
        printf("FMOD error! (%d) %s\n", result, FMOD_ErrorString(result));
        exit(-1);
     }
}

//------------------------Global variables     

//---------------CAMERA GLOBALS--------------------
float cam_zoom = 15.0;
float cam_rotx = 0;
float cam_roty = 0.001;
float cam_tx = 0;
float cam_ty = 0;
int cam_lastx=0;
int cam_lasty=0;
int mouseButtons[3] = {0, 0, 0};

// move light globals
float lightRot[4] = {0.0, 0.0, 0.0, 0.0};
float velocity[4] = {0, 0, 0, 0};
bool upPressed = false;           
bool downPressed = false;
bool leftPressed = false;
bool rightPressed = false;
float velocityFactor = 0.008;
float timer;
float frameStart = -1;
float frameEnd = 0;

float lightPosXLast = 0;
float lightPosYLast = 0;
float lightPosZLast = 0;

// Variables for 3D sound listener
int listenerflag = TRUE;
FMOD_VECTOR listenerpos  = { 0.0f, 0.0f, 0.0f };
FMOD_VECTOR forward = { 0.0f, 0.0f, 1.0f };
FMOD_VECTOR up = { 0.0f, 1.0f, 0.0f };
FMOD_VECTOR listenervel = { 0.0f, 0.0f, 0.0f };

class SoundSource;
SoundSource *SoundSourceArray[50];

// Creating a class for each sound was inspired by the fmod example 'virtual voices'
class SoundSource
{
    private:
    FMOD_CHANNEL *channel3d ;
    FMOD_VECTOR pos, vel, velocity;
    bool active;
        
    public:
    SoundSource(float roty, float rotz, int randSound)
    {
        glLoadIdentity();
        glRotatef(roty,0,1,0);
        glRotatef(rotz,0,0,1);
        glTranslatef(10,0,0);
        float soundMatrix[16];
        glGetFloatv(GL_TRANSPOSE_MODELVIEW_MATRIX, soundMatrix);
        Mat4 soundWorldSpaceBase(soundMatrix[0], soundMatrix[1], soundMatrix[2], soundMatrix[3],
                                 soundMatrix[4], soundMatrix[5], soundMatrix[6], soundMatrix[7],
                                 soundMatrix[8], soundMatrix[9], soundMatrix[10], soundMatrix[11],
                                 soundMatrix[12], soundMatrix[13], soundMatrix[14], soundMatrix[15]);
                           
        Vec4 ident(0.0, 0.0, 0.0, 1.0);
        Vec4 soundVec = soundWorldSpaceBase * ident;
            
        pos.x = soundVec[0];
        pos.y = soundVec[1];
        pos.z = soundVec[2];
        vel.x = 0.0f;
        vel.y = 0.0f;
        vel.z = 0.0f;

        printf("pos.x = %f\n",pos.x);
        printf("pos.y = %f\n",pos.y);
        printf("pos.z = %f\n",pos.z);
        
        channel3d = channels[randSound];

        result = FMOD_System_PlaySound(fmodSystem, FMOD_CHANNEL_FREE, sounds[randSound], true, &channel3d);
        ERRCHECK(result);
        
        result = FMOD_Channel_Set3DAttributes(channel3d, &pos, &vel);
        ERRCHECK(result);
        
        result = FMOD_Channel_SetPaused(channel3d, false);
        ERRCHECK(result);
    }
    
    void DrawSound()
    {
         if(active)
         {
                     glColor3f(1.0,1.0,1.0);
         }
         else
         {
                     glColor3f(0.0,1.0,0.0);
         }
         
   	           glPushMatrix();
                   glTranslatef(pos.x,pos.y,pos.z);
                   glScalef(0.2,0.2,0.2);
    		       glBegin(GL_TRIANGLE_FAN);
                       glVertex3f(0,1,0);
            	       glVertex3f(-0.5,0,-0.5);
        	           glVertex3f(-0.5,0,0.5);
            	       glVertex3f(0.5,0,0.5);
            	       glVertex3f(0.5,0,-0.5);
    	           glEnd();
  	           glPopMatrix();
    }
    
    float isLive(float leastDistance)
    {
                float dx = listenerpos.x-pos.x;
                float dy = listenerpos.y-pos.y;
                float dz = listenerpos.z-pos.z;                
                float distance = sqrt(dx*dx + dy*dy + dz*dz);                
                return distance; 
    }
    
    void deactivate()
    {
         active = false;
    }
    
    void activate()
    {
         active = true;
    }  
};

void updateListener(float timeDiff)
{
      glLoadIdentity(); 
      glRotatef(lightRot[0],0,1,0);
      glRotatef(lightRot[1],0,0,1);
      glTranslatef(10,0,0);
      float lightMatrix[16];
      glGetFloatv(GL_TRANSPOSE_MODELVIEW_MATRIX, lightMatrix);
      Mat4 listenerWorldSpaceBase(lightMatrix[0], lightMatrix[1], lightMatrix[2], lightMatrix[3],
                                  lightMatrix[4], lightMatrix[5], lightMatrix[6], lightMatrix[7],
                                  lightMatrix[8], lightMatrix[9], lightMatrix[10], lightMatrix[11],
                                  lightMatrix[12], lightMatrix[13], lightMatrix[14], lightMatrix[15]);
                               
      Vec4 ident(0.0, 0.0, 0.0, 1.0);                       
      Vec4 lightVec = listenerWorldSpaceBase * ident;    
      float lightPos[4] = {lightVec[0], lightVec[1], lightVec[2], 0.0};
      
      // Update the listener
      listenerpos.x = lightPos[0];
      listenerpos.y = lightPos[1];
      listenerpos.z = lightPos[2];     
    
      listenervel.x = (lightPos[0] - lightPosXLast) * 1000 / timeDiff;
      listenervel.y = (lightPos[1] - lightPosYLast) * 1000 / timeDiff;
      listenervel.z = (lightPos[2] - lightPosZLast) * 1000 / timeDiff; 
      
      lightPosXLast = lightPos[0];   
      lightPosYLast = lightPos[1];
      lightPosZLast = lightPos[2];
      
      result = FMOD_System_Set3DListenerAttributes(fmodSystem, 0, &listenerpos, &listenervel, &forward, &up);
      ERRCHECK(result);
}    

void updateSounds()
{
        float leastDistance = 20;
        int store;
        
        // Calculate which sound is closest
        for(int count=0; count<NUMSOUNDS; count++)
        {        
              if(SoundSourceArray[count]->isLive(leastDistance)<leastDistance)
              {
                                        leastDistance = SoundSourceArray[count]->isLive(count);
                                        store = count;
              }
        }
        
        for(int count=0; count<NUMSOUNDS; count++)
        {
              if(count==store) continue;
              SoundSourceArray[count]->deactivate();
        }
        
        SoundSourceArray[store]->activate();
}

//-------------------------------------------------
void motion(int x,int y)
{
	int diffx=x-cam_lastx;
	int diffy=y-cam_lasty;
	cam_lastx=x;
	cam_lasty=y;

	if( mouseButtons[2])
	{
		cam_zoom -= (float) 0.05f * diffx;
	}
	else
		if( mouseButtons[0] )
		{
			cam_rotx += (float) 0.5f * diffy;
			cam_roty += (float) 0.5f * diffx;	
		}
		else
			if( mouseButtons[1] )
			{
				cam_tx += (float) 0.05f * diffx;
				cam_ty -= (float) 0.05f * diffy;
			}
}

void moveLight(float timeDiff)
{          
           //accelerate
           if(upPressed) velocity[0]+=(velocityFactor*timeDiff);
           if(downPressed) velocity[1]+=(velocityFactor*timeDiff);                     
           if(leftPressed) velocity[2]+=(velocityFactor*timeDiff);           
           if(rightPressed) velocity[3]+=(velocityFactor*timeDiff);           
                      
           //decelerate
           if(!upPressed) velocity[0]-=(velocityFactor*timeDiff);
           if(!downPressed) velocity[1]-=(velocityFactor*timeDiff);
           if(!leftPressed) velocity[2]-=(velocityFactor*timeDiff);
           if(!rightPressed) velocity[3]-=(velocityFactor*timeDiff);
           
           for(int i=0;i<4;i++)
           {
               if(velocity[i]>4) velocity[i] = 4;
               if(velocity[i]<0) velocity[i] = 0;
           }
           
           //update position        
           lightRot[1] += (0.1 * velocity[0]);
           lightRot[1] -= (0.1 * velocity[1]);
           lightRot[0] -= (0.1 * velocity[2]);
           lightRot[0] += (0.1 * velocity[3]);
                      
           if(lightRot[1]<0) lightRot[1]=0;
           if(lightRot[1]>85) lightRot[1]=85; 
}

//--------------------------------------------------------------------------------------DISPLAY
void display()
{
     glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
     
     float angle;
	
	glLoadIdentity();
	glTranslatef(0,0,-cam_zoom);
	glTranslatef(cam_tx,cam_ty,0);
	glRotatef(cam_rotx,1,0,0);
	glRotatef(cam_roty,0,1,0);	
	
	glColor3f(1.0,1.0,1.0);
	
	// draw grid
	glBegin(GL_LINES);
	for(int i=-10;i<=10;++i) {
		glVertex3f(i,0,-10);
		glVertex3f(i,0,10);

		glVertex3f(10,0,i);
		glVertex3f(-10,0,i);
	}
	glEnd();
	
	glColor3f(1.0,0.0,0.0);
	
	// Draw where FMOD thinks the listener is
	glPushMatrix();
        glTranslatef(listenerpos.x,listenerpos.y,listenerpos.z);
       	glBegin(GL_POLYGON);
	    for(int i =0;i<10;i++)
	    {
            angle=2*PI*i/10;
            glVertex3f(cos(angle),0,sin(angle));
        }
        glEnd();
	glPopMatrix();
	
	glColor3f(0.0,1.0,0.0);
  	
    // Draw the sounds
    for (int count=0;count<NUMSOUNDS;count++)
    {
        SoundSourceArray[count]->DrawSound();
    } 
    
    SDL_GL_SwapBuffers();
}

//------------------------------------------------------------------------------------------MAIN
int main(int argc, char *argv[])
{
    
    int active = 0;
    char *mystrings[17] = {"../gameSounds/gameSounds1.aif","../gameSounds/gameSounds2.aif","../gameSounds/gameSounds3.aif",
                           "../gameSounds/gameSounds4.aif","../gameSounds/gameSounds5.aif","../gameSounds/gameSounds6.aif",
                           "../gameSounds/gameSounds7.aif","../gameSounds/gameSounds8.aif","../gameSounds/gameSounds9.aif",
                           "../gameSounds/gameSounds10.aif","../gameSounds/gameSounds11.aif","../gameSounds/gameSounds12.aif",
                           "../gameSounds/gameSounds13.aif","../gameSounds/gameSounds14.aif","../gameSounds/gameSounds15.aif",
                           "../gameSounds/gameSounds16.aif","../gameSounds/gameSounds17.aif"};
         		
	// Create the main system object
    result = FMOD_System_Create(&fmodSystem);
    ERRCHECK(result);
    
    result = FMOD_System_SetSoftwareChannels(fmodSystem, 5);
    ERRCHECK(result);
     
    // Initialise FMOD to default settings
	result = FMOD_System_Init(fmodSystem, 1000, FMOD_INIT_NORMAL, 0);
	ERRCHECK(result);
	
	// Create DSP effects
    //result = FMOD_System_CreateDSPByType(fmodSystem, FMOD_DSP_TYPE_ECHO, &dspflange);
    //ERRCHECK(result);                
    //result = FMOD_System_AddDSP(fmodSystem, dspflange);
    //ERRCHECK(result);
    
   	// 3D settings -   float  dopplerscale, float  distancefactor, float  rolloffscale
    result = FMOD_System_Set3DSettings(fmodSystem, 1.0, 1.0, 1.0);
    ERRCHECK(result);
	
	// Setup all the 3D sounds
	for(int count=0; count<NUMCLIPS; count++)
	{
        result = FMOD_System_CreateSound(fmodSystem, mystrings[count], FMOD_SOFTWARE | FMOD_3D | FMOD_LOOP_NORMAL, 0, &sounds[count]);
        ERRCHECK(result);
        result = FMOD_Sound_Set3DMinMaxDistance(sounds[count], 1.0f, 1000.0f);
        ERRCHECK(result);
        result = FMOD_Sound_SetMode(sounds[count], FMOD_LOOP_NORMAL);
        ERRCHECK(result);
    }
            
  // Initialise SDL ------------------------------------------------------------  
  SDL_Event event;
  
  SDL_Init(SDL_INIT_VIDEO);
  SDL_WM_SetCaption  ("lightGrow", "lightGrow.ico");
  SDL_SetVideoMode(1200, 600, 0, SDL_OPENGL | SDL_HWSURFACE);

  Uint32 timeNow;
  Uint32 timeThen = 0;

  srand ( time(NULL) );

  glViewport(0, 0, 1200, 600);
  glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
  glClearDepth(1.0);
  glDepthFunc(GL_LESS);

//-----------------------enables
	glEnable(GL_DEPTH_TEST);
	
  glMatrixMode(GL_PROJECTION);
  glFrustum(-0.08284,0.08284,-0.04142,0.04142,0.1,100);
  glMatrixMode(GL_MODELVIEW);

    // Initialise the sound sources
    for (int count=0;count<NUMSOUNDS;count++)
    {
        int y = rand() % 361;
        int z = rand() % 86;
        int randSound = rand() % 17;

        printf("int y = %d\n",y);
        printf("int z = %d\n",z);
        printf("randSound = %d\n",randSound);

        SoundSourceArray[count] = new SoundSource((float)y, (float)z, randSound);
    }
  

//---------------------------------------------------------------------------------------------------------
//----------------------------------------  MASTER LOOP  --------------------------------------------------
//---------------------------------------------------------------------------------------------------------
  bool done = false;
  while(!done)
  {
           printf("\n NEW FRAME \n\n");
           while(SDL_PollEvent(&event)) 
           {
           switch(event.type){
                              case SDL_MOUSEBUTTONDOWN:
                                   if(event.button.button == SDL_BUTTON_LEFT){
                                                   mouseButtons[0] = 1;
                                                   }
                                   else if(event.button.button == SDL_BUTTON_MIDDLE){
                                                   mouseButtons[1] = 1;
                                                   }
                                   else if(event.button.button == SDL_BUTTON_RIGHT){
                                                   mouseButtons[2] = 1;
                                                   }        
                                                   cam_lastx = event.button.x;
                                                   cam_lasty = event.button.y;
                                                   break;
                                                   
                              case SDL_MOUSEBUTTONUP:
                                   if(event.button.button == SDL_BUTTON_LEFT){
                                                   mouseButtons[0] = 0;
                                                   }
                                   else if(event.button.button == SDL_BUTTON_MIDDLE){
                                                   mouseButtons[1] = 0;
                                                   }
                                   else if(event.button.button == SDL_BUTTON_RIGHT){
                                                   mouseButtons[2] = 0;
                                                   }
                                                   break;
                                                   
                              case SDL_MOUSEMOTION:
                                   motion(event.motion.x, event.motion.y);
                                   break;
                                   
                              case SDL_KEYDOWN:
                                   if(event.key.keysym.sym == SDLK_UP){
                                                   upPressed = true;
                                                   }
                                   if(event.key.keysym.sym == SDLK_DOWN){
                                                   downPressed = true;
                                                   }
                                   else if(event.key.keysym.sym == SDLK_LEFT){
                                                   leftPressed = true;
                                                   }
                                   else if(event.key.keysym.sym == SDLK_RIGHT){
                                                   rightPressed = true;
                                                   }
                                                   break;
                                                   
                              case SDL_KEYUP:
                                   if(event.key.keysym.sym == SDLK_UP){
                                                   upPressed = false;
                                                   }
                                   if(event.key.keysym.sym == SDLK_DOWN){
                                                   downPressed = false;
                                                   }
                                   else if(event.key.keysym.sym == SDLK_LEFT){
                                                   leftPressed = false;
                                                   }
                                   else if(event.key.keysym.sym == SDLK_RIGHT){
                                                   rightPressed = false;
                                                   }
                                                   break;                                  
                                                   
                              case SDL_QUIT:
                                   done = true;
                                   break;
                                   
                                   }
           }       
  
  timeNow = SDL_GetTicks();
  float timeDiff = timeNow - timeThen;
  timeThen = timeNow;
  
  moveLight(timeDiff);  
  updateListener(timeDiff);
  updateSounds();
       
  // Update FMOD    
  FMOD_System_Update(fmodSystem); 
   
  //-----------------display a frame now!-------------------------
  glLoadIdentity();  
  display();
    
  }
//---------------------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------------------

    // Exit FMOD
    for(int count=0;count<NUMCLIPS;count++)
    {
        result = FMOD_Sound_Release(sounds[count]);
	    ERRCHECK(result);
    }
    result = FMOD_System_Close(fmodSystem);
    ERRCHECK(result);
    result = FMOD_System_Release(fmodSystem);
    ERRCHECK(result);

  SDL_Quit();
  return(0);
}
