//********************************************************
//*     _  _       _          _______                    *
//*    | |(_)     | |     _  (_______)                   *
//*    | | _  ____| |__ _| |_ _   ___  ____ ___  _ _ _   *
//*    | || |/ _  |  _ (_   _) | (_  |/ ___) _ \| | | |  *
//*    | || ( (_| | | | || |_| |___) | |  | |_| | | | |  *
//*    \_)_|\___ |_| |_| \__)\_____/|_|   \___/ \___/    *
//*        (_____|                                       *
//*                                                      * 
//*******************************************************
//  By
//  Patrick Anderson        &         Thomas Stratford  
//                                                2007
//            




// includes -----------------------
#include <SDL/SDL.h>
//#include <gl/glew.h>
#include <gl/glut.h>
#include <gl/glext.h>

#include<svl/SVL.h>
#include <svl/SVLgl.h>

#include <iostream>
#include <math.h>
#include <vector>
#include <stack>

#include <fmod.h>
#include <fmod_errors.h>
//-----------------------------------

// From FMOD example "pitchdetection"
#define OUTPUTRATE          44100
#define SPECTRUMSIZE        8192
#define SPECTRUMRANGE       ((float)OUTPUTRATE / 2.0f)      /* 0 to nyquist */
#define BINSIZE      (SPECTRUMRANGE / (float)SPECTRUMSIZE)

// FMOD Errorcheck -------------------------------------------------------------
void ERRCHECK(FMOD_RESULT result)
{
     if (result != FMOD_OK)
     {
        printf("FMOD error! (%d) %s\n", result, FMOD_ErrorString(result));
        exit(-1);
     }
}

//-------------------------------------------------------------------------------------------------MULT MATRIX BY VECTOR
void matrixByVector(float *matrix, float *vector, float *store)
 {
	store[0] = (vector[0] * matrix[0]) + (vector[1] * matrix[1]) + (vector[2] * matrix[2]) + (vector[3] * matrix[3]);
	store[1] = (vector[0] * matrix[4]) + (vector[1] * matrix[5]) + (vector[2] * matrix[6]) + (vector[3] * matrix[7]);
	store[2] = (vector[0] * matrix[8]) + (vector[1] * matrix[9]) + (vector[2] * matrix[10]) + (vector[3] * matrix[11]);
	store[3] = (vector[0] * matrix[12]) + (vector[1] * matrix[13]) + (vector[2] * matrix[14]) + (vector[3] * matrix[15]);
}

        
typedef struct vertex{
              float x;
              float y;
              float z;
      }vert;

// Stack to track memory allocation.
// This holds pointers to all the blocks of  memory so they
// can be deleted.
std::stack<void*> plantMemory;

//-------------------------------------------------------------------------------------------------BRANCH NODE CLASS
class branch
{
      private:
              int generation;
              float length;
              float finalLength;
              Uint32 birthTime; //ticks on creation
              float radius;
              int splits;
              float rotX, rotY, rotZ;
              float birthRotX, birthRotY, birthRotZ;
              float bendMax;
              std::vector<branch*> child;
              branch *parent;
              vert vertexRing[6];
              vert normalRing[6];
              bool root;
              bool stop;
              
      
      public:
             float getLength()
             {return length;}
             float getFinalLength()
             {return finalLength;}
             float getBendMax()
             {return bendMax;}
             
             void setLength(float length_in)
             {length = length_in;}
             
             float getRotX()
             {return rotX;}
             float getRotY()
             {return rotY;}
             float getRotZ()
             {return rotZ;}
             
             void setRotX(float rotX_in)
             {rotX = rotX_in;}
             void setRotY(float rotY_in)
             {rotY = rotY_in;}
             void setRotZ(float rotZ_in)
             {rotZ = rotZ_in;}
             
             branch(bool root_in, int generation_in, float radius_in, float initialLength, float finalLength_in, branch *parent_in,
              float rotX_in, float rotY_in, float rotZ_in, float bendMax_in, int splits_in){
                          generation = generation_in;
                          rotX = rotX_in;
                          rotY = rotY_in;
                          rotZ = rotZ_in;
                          birthRotX = rotX_in;
                          birthRotY = rotY_in;
                          birthRotZ = rotZ_in;
                          length = initialLength;
                          radius = radius_in;
                          parent = parent_in;
                          birthTime = SDL_GetTicks();
                          finalLength = finalLength_in;
                          bendMax = bendMax_in;
                          splits = splits_in;
                          root = root_in;
                          stop = false;
                          }
                          
             branch(){
                      //printf("Blank node.\n");
                      }
                          
             void setChild (branch *child_in){
                  child.push_back(child_in);
                  }
                     
             branch* getParent(){
                     return parent;
                     }
                  
             vert* getVertexRing(){
                     return vertexRing;
                     }
                     
             vert* getNormalRing(){
                   return normalRing;
                   }
                   
             //----------------------------------------------------------------------------
             //------------ function to navigate and draw the plant -----------------------
             //----------------------------------------------------------------------------   
              
             void drawPlant(){
                              vert* ringCurrent; vert* ringCurrentNormals;
                              vert* ringLast; vert* ringLastNormals;
                              
                              
                               // base ring temporary setup
                                 vert baseRing[6];
                                 if (parent == NULL){
                                         //-----------loop around the ring ---------------------------
                                             for(int segmentCount = 0; segmentCount < 6; segmentCount++){
                                                     
                                                     float sectionAngle = (segmentCount*(6.2831853)/6);
                                                     float xOffset = radius*(cos(sectionAngle));
        			                                 float zOffset = radius*(sin(sectionAngle));
        			                                 
        			                                 if(!root){
                                                               baseRing[segmentCount].x = xOffset;
                                                               baseRing[segmentCount].y = 0;
                                                               baseRing[segmentCount].z = zOffset;
                                                               }
                                                      else{
                                                           baseRing[((5-segmentCount)+4)%6].x = xOffset;
                                                           baseRing[((5-segmentCount)+4)%6].y = 0;
                                                           baseRing[((5-segmentCount)+4)%6].z = zOffset;  
                                                           }       
                                                     }
      
                                            }
                                            
                             // retrive the vertex rings         
                               ringCurrent = vertexRing;
                               if (parent != NULL){
                                                  ringLast = parent->getVertexRing();
                                                  }
                               else {ringLast = baseRing;}
                               
                               //retrive the normal rings
                               ringCurrentNormals = normalRing;
                               if (parent != NULL){
                                                  ringLastNormals = parent->getNormalRing();
                                                  }
                               else {ringLastNormals = ringCurrentNormals;}
                               
                               
                               //draw the polygons
                     	       glBegin(GL_QUAD_STRIP);
                                  for(int segCount = 0; segCount < 6; segCount++){
                                        float x1 = ringLast[segCount].x;
                                        float y1 = ringLast[segCount].y;
                                        float z1 = ringLast[segCount].z;
                                        float x2 = ringCurrent[segCount].x;
                                        float y2 = ringCurrent[segCount].y;
                                        float z2 = ringCurrent[segCount].z;
                                        float n1_x = ringLastNormals[segCount].x;
                                        float n1_y = ringLastNormals[segCount].y;
                                        float n1_z = ringLastNormals[segCount].z;
                                        float n2_x = ringCurrentNormals[segCount].x;
                                        float n2_y = ringCurrentNormals[segCount].y;
                                        float n2_z = ringCurrentNormals[segCount].z;
                                        
                                        glNormal3f(n1_x, n1_y, n1_z);
                                        glVertex3f(x1, y1, z1);
                                        glNormal3f(n2_x, n2_y, n2_z);
                                        glVertex3f(x2, y2, z2);
                                        }
                                        
                                        //and once more to close the ring
                                        float x1 = ringLast[0].x;
                                        float y1 = ringLast[0].y;
                                        float z1 = ringLast[0].z;
                                        float x2 = ringCurrent[0].x;
                                        float y2 = ringCurrent[0].y;
                                        float z2 = ringCurrent[0].z;
                                        float n1_x = ringLastNormals[0].x;
                                        float n1_y = ringLastNormals[0].y;
                                        float n1_z = ringLastNormals[0].z;
                                        float n2_x = ringCurrentNormals[0].x;
                                        float n2_y = ringCurrentNormals[0].y;
                                        float n2_z = ringCurrentNormals[0].z;
                                        
                                        glNormal3f(n1_x, n1_y, n1_z);
                                        glVertex3f(x1, y1, z1);
                                        glNormal3f(n2_x, n2_y, n2_z);
                                        glVertex3f(x2, y2, z2);
                                        
                                            
                                glEnd();
                                
                                //recursivly perfom this operation on its children
                                std::vector<branch*>::iterator it = child.begin();
                                   for(; it != child.end(); ++it){
                                         (*it)->drawPlant();
                                         }
                  }
             //---------------------------------------------------------------------------- 
             //---------------------------------------------------------------------------- 
             
                 
             //----------------------------------------------------------------------------
             //-------------------- function to update the vertex rings -------------------
             //----------------------------------------------------------------------------      
                    
             void updateVertexRing(){
                                    
                                    // perform the translations
                                    glRotatef(rotX,1,0,0);
                                    glRotatef(rotY,0,1,0);
                                    glRotatef(rotZ,0,0,1);
                                    glTranslatef(0,length,0);
                                    
                                    //update the ring values
                                     float currentMatrix[16];
                                     float sectionAngle;
                                     float xOffset, zOffset;
                                     float worldV1[4];
                                     float ringCenter[4];
                                     vert worldV, worldN;
                                     //-------load a copy of the world space transformation-------------
	                                 glGetFloatv(GL_TRANSPOSE_MODELVIEW_MATRIX, currentMatrix);
	                                 //-----------loop around the ring ---------------------------
                                     for(int segmentCount = 0; segmentCount < 6; segmentCount++){
                                             
                                             sectionAngle = (segmentCount*(6.2831853)/6);
                                             xOffset = radius*(cos(sectionAngle));
			                                 zOffset = radius*(sin(sectionAngle));
			                                 
			                                 //-------calculate the world-space coords----------------
                                             float v1[4] = {xOffset, 0.0, zOffset, 1};
		                                  	 matrixByVector(currentMatrix, v1, worldV1);
		                                  	 
		                                  	 //---------------calculate the normal----------------
		                                  	 float identity[4] = {0.0, 0.0, 0.0, 1.0};
		                                  	 matrixByVector(currentMatrix, identity, ringCenter);
		                                  	 Vec3 worldN1((worldV1[0]-ringCenter[0]),
                                                                (worldV1[1]-ringCenter[1]),
                                                                 (worldV1[2]-ringCenter[2]));
                                             worldN1 = norm(worldN1);
                                                                
		                                  	 //-------update the vertex in the ring----------
		                                  	 worldV.x = worldV1[0];   worldV.y = worldV1[1];   worldV.z = worldV1[2];
		                                  	 vertexRing[segmentCount] = worldV;
		                                  	 //and the normal
		                                  	 worldN.x = worldN1[0];   worldN.y = worldN1[1];   worldN.z = worldN1[2];
		                                  	 normalRing[segmentCount] = worldN;
			                                
                                             }
                                   
                                   //if this is a leaf
                                   if(child.empty()){
                                                   glPopMatrix();
                                                   }
                                   else{ 
                                       //if this is a split point
                                       if(child.size() > 1){
                                                       //recurse for all its children
                                                       std::vector<branch*>::iterator it = child.begin();
                                                       for(; it != child.end(); ++it){
                                                             glPushMatrix();
                                                             (*it)->updateVertexRing();
                                                             }
                                                       glPopMatrix();
                                                       }
                                       //otherwise its just a normal branch node 
                                       else{
                                            //recurse for its child
                                            std::vector<branch*>::iterator it = child.begin();
                                            (*it)->updateVertexRing();
                                            }
                                       
                                       }
                                   
                                   }
             //---------------------------------------------------------------------------- 
             //---------------------------------------------------------------------------- 
             
                                   
             //----------------------------------------------------------------------------
             //------------ function to apply growth towards the light --------------------
             //---------------------------------------------------------------------------- 
             
             void phototropism(Vec4 lightPos, float bend, float growth){
                  
                  //apply the increse in length
                  if( root ){
                      if ( (parent == NULL) || (parent->length == parent->finalLength) ){
                           length = length + (growth*10);
                           if (length > finalLength)length = finalLength;
                           }
                      }
                      
                  else{
                       length = length + growth;
                       if (length > finalLength)length = finalLength;
                       }
                  
                  //------branch rotation towards light
                  
                  // perform the translations
                  glPushMatrix();
                  glRotatef(rotX,1,0,0);
                  glRotatef(rotY,0,1,0);
                  glRotatef(rotZ,0,0,1);
                  
                        //find the lights position in local space 
                        float currentMatrix[16];
                        glGetFloatv(GL_TRANSPOSE_MODELVIEW_MATRIX, currentMatrix);
                        Mat4 worldSpaceBase(currentMatrix[0], currentMatrix[1], currentMatrix[2], currentMatrix[3],
                                             currentMatrix[4], currentMatrix[5], currentMatrix[6], currentMatrix[7],
                                              currentMatrix[8], currentMatrix[9], currentMatrix[10], currentMatrix[11],
                                               currentMatrix[12], currentMatrix[13], currentMatrix[14], currentMatrix[15]);
                                               
                        Mat4 localSpaceBase = inv(worldSpaceBase);
                        Vec4 localLightPos = localSpaceBase * lightPos;
                        
                        //find the diffrence in direction of the branch to the direction of the light
                        Vec2 lightZY(localLightPos[2],localLightPos[1]);
                        Vec2 lightXY(localLightPos[0],localLightPos[1]);
                        Vec2 branchDir(0.0, 1.0);
                        
                        float lightRotX = acos( (dot(lightZY, branchDir))/
                                                (len(lightZY) * len(branchDir)) );
                        float lightRotZ = acos( dot(lightXY, branchDir)/
                                                (len(lightXY) * len(branchDir))); 
                                                
                        //convert to degrees                        
                        lightRotX = lightRotX * 57.2957795;
                        lightRotZ = lightRotZ * 57.2957795;
                        
                        //rotate the branch the correct amount toward the light
                        //---------handle around X
                        if(lightRotX > 1)
                        {
                            float bendX = bend;
                            if(root)bendX = (bendMax-1);
                            if (bendX > lightRotX) bendX = lightRotX;
                            if (localLightPos[2] < 0) bendX = (0.0 - bendX);
                            float newRotX = (rotX + bendX);
                            //limit the bend amount
                            if(  (newRotX <= (birthRotX + bendMax)) && (newRotX >= (birthRotX - bendMax))  ){
                                 rotX = newRotX;
                                 }
                        }
                        
                        //---------handle around Z
                        if(lightRotZ > 1)
                        {
                            float bendZ = bend;
                            if(root)bendZ = (bendMax-1);
                            if (bendZ > lightRotZ) bendZ = lightRotZ;
                            if (localLightPos[0] > 0) bendZ = (0.0 - bendZ);
                            float newRotZ = (rotZ + bendZ);
                            //limit the bend amount
                            if(  (newRotZ <= (birthRotZ + bendMax)) && (newRotZ >= (birthRotZ - bendMax))  ){
                                 rotZ = newRotZ;
                                 }
                                         
                        }
                      //-------------------------------------------------------------------------
                  //reset with the new translations
                  glPopMatrix();
                  glRotatef(rotX,1,0,0);
                  glRotatef(rotY,0,1,0);
                  glRotatef(rotZ,0,0,1);
                  glTranslatef(0,length,0);
                  //------------------------------------------------------------

                 
                  // Generate new growth if needed
                  int maxGenerations;
                  if (root) maxGenerations = 2;
                  else maxGenerations = 3;
                  if( (splits > 0) && (child.size() < 2) && (length > (finalLength / 2)) && (child.size() > 0) && (generation < maxGenerations) ){
                      
                      for(int i=0; i<splits; i++)
                      {
                              
                          //---------------------------make some branch nodes---------------------
                            branch *newLastBranch_ptr = this;
                            
                            float newInitialLength = 0.0;
                            float newFinalLength = finalLength / 2; 
                            if(root && parent == NULL){
                                    newFinalLength = finalLength; 
                                    generation = 0;
                                    }
                            else if(root) newFinalLength = finalLength / 3; 
                            float newRadius = radius * 0.8;
                            
                            branch *newBranch;
                            newBranch = new branch [10];
                            plantMemory.push(newBranch);
                            
                            for(int i=0; i < 10; i++){
                                    //branch newBranch;
                                        int newSplits = 0;
                                        float newBendMax = 5;
                                        if (root) newBendMax = 10;
                                        float newRotX = (-10+rand()%20);
                                        float newRotY = 0;
                                        float newRotZ = (-10+rand()%20);
                                        
                                        if( i == 0 ){
                                            newBendMax = 2;
                                            newRotX = (-70+rand()%140);
                                            if ( abs(newRotX) > 50 ) newRotZ = (-70+rand()%140);
                                            else {
                                                 newRotZ = (50+rand()%20);
                                                 if((rand()%20) > 10){
                                                               newRotZ = (0 - newRotZ);
                                                               }
                                                 }
        
                                            }
                                        
                                        if ( (i+1)%2 == 0 ) newSplits = 2;
                                        
                                        //---------initialize the node
                                        newBranch[i] = branch(root, (generation + 1), newRadius, newInitialLength, newFinalLength, newLastBranch_ptr, newRotX, newRotY, newRotZ, newBendMax, newSplits);
                                        //---------set its parent to point to it
                                        newLastBranch_ptr->setChild(&newBranch[i]);
                                        //---------set the last branch to be this one                          
                                        newLastBranch_ptr = &newBranch[i];
                                        
                                        }
    
                                    }
                           }
                  
                  
                  
                  
                  //if this is a leaf
                   if(child.empty()){
                                   glPopMatrix();
                                   }
                   else{ 
                       //if this is a split point
                       if(child.size() > 1){
                                       //recurse for all its children
                                       std::vector<branch*>::iterator it = child.begin();
                                       for(; it != child.end(); ++it){
                                             glPushMatrix();
                                             (*it)->phototropism(lightPos, bend, growth);
                                             }
                                       glPopMatrix();
                                       }
                       //otherwise its just a normal branch node 
                       else{
                            //recurse for its child
                            std::vector<branch*>::iterator it = child.begin();
                            (*it)->phototropism(lightPos, bend, growth);
                            }
                       
                       }
                  
                  
                  }
                  //---------------------------------------------------------------------------- 
                  //---------------------------------------------------------------------------- 
                             
           
};
//-------------------------------------------------------------------------------------------------------------------------

//------------------------Global variables      
branch *stem_ptr;
branch *root_ptr;

// move light globals
float lightRot[2] = {0.0, 0.0};
float velocity[4] = {0, 0, 0, 0};
bool upPressed = false;           
bool downPressed = false;
bool leftPressed = false;
bool rightPressed = false;
float velocityFactor = 0.0001;

//FMOD globals

FMOD_SYSTEM *fmodSystem = 0;
FMOD_SOUND *sound = 0;
FMOD_CHANNEL *channel = 0;
FMOD_RESULT result;
FMOD_CREATESOUNDEXINFO exinfo;

//mic variables

//input sound variables
float maxVolNow = 0;
float maxVolThen = 0;
float freqNow = 0;
float freqThen = 0;
float volumeAverage = 0;
float frequencyAverage = 0;
float updateTimer = 0;
int numSeconds = 0;
int numValues = 0; // Used for calculating the average volume and frequency

//----------ROB BATEMAN'S CAMERA CODE--------------------------*****<ROBS CODE>******

//---------------CAMERA GLOBALS--------------------
float cam_zoom = 60.0;
float cam_rotx = 27;
float cam_roty = 27;
float cam_tx = 0;
float cam_ty = 0;
int cam_lastx=0;
int cam_lasty=0;
int mouseButtons[3] = {0, 0, 0};
//-------------------------------------------------

void motion(int x,int y)
{
	int diffx=x-cam_lastx;
	int diffy=y-cam_lasty;
	cam_lastx=x;
	cam_lasty=y;

	if( mouseButtons[2])
	{
		cam_zoom -= (float) 0.05f * diffx;
	}
	else
		if( mouseButtons[0] )
		{
			cam_rotx += (float) 0.5f * diffy;
			cam_roty += (float) 0.5f * diffx;	
		}
		else
			if( mouseButtons[1] )
			{
				cam_tx += (float) 0.05f * diffx;
				cam_ty -= (float) 0.05f * diffy;
			}
}

//---------------------------------------------------******</ROBS CODE>******

//---------------------------------------------------------------window resize
void resize (int width, int height){
     if(height == 0)height = 1;
     glViewport(0, 0, width, height);
       glClearColor(0.3f, 0.3f, 0.4f, 0.0f);
       
    //----------------------------------------------------------------       
      glClearDepth(1.0);
      glDepthFunc(GL_LESS);
    //-----------------------setup the materials and shading
    	GLfloat mat_specular[] = {0.1, 0.2, 0.1, 0.3};
    	GLfloat mat_shininess[] = {100.0};
    	GLfloat light_position[] = {10, 10, 10, 1.0};
    	GLfloat white_light[] = {1.0, 1.0, 1.0, 1.0};
    	GLfloat lmodel_ambient[] = {0.1, 0.1, 0.1, 1.0};
    
    	glShadeModel(GL_SMOOTH);
    	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
    	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, mat_shininess);
    	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
    	glLightfv(GL_LIGHT0, GL_DIFFUSE, white_light);
    	glLightfv(GL_LIGHT0, GL_SPECULAR, white_light);
    	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
        glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
    
    //-----------------------enables
    	glEnable(GL_LIGHTING);
    	glEnable(GL_LIGHT0);
    	glEnable(GL_DEPTH_TEST);
    //---------------------------------------------------------------
    
     glMatrixMode(GL_PROJECTION);
     glLoadIdentity();
     gluPerspective(45,(float)width/height,0.1,200);
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity();
    	
     
     
     }

//---------------------------------------------------------------move light
void moveLight(float timeDiff)
{
                
           //accelerate
           if(upPressed) velocity[0]+=(velocityFactor*timeDiff);
           if(downPressed) velocity[1]+=(velocityFactor*timeDiff);                     
           if(leftPressed) velocity[2]+=(velocityFactor*timeDiff);           
           if(rightPressed) velocity[3]+=(velocityFactor*timeDiff);           
                      
           //decelerate
           if(!upPressed) velocity[0]-=(velocityFactor*timeDiff);
           if(!downPressed) velocity[1]-=(velocityFactor*timeDiff);
           if(!leftPressed) velocity[2]-=(velocityFactor*timeDiff);
           if(!rightPressed) velocity[3]-=(velocityFactor*timeDiff);
           
           for(int i=0;i<4;i++)
           {
               if(velocity[i]>0.1) velocity[i] = 0.1;
               if(velocity[i]<0) velocity[i] = 0;
           }
           
           //update position           
           lightRot[1] += (timeDiff * velocity[0]);
           lightRot[1] -= (timeDiff * velocity[1]);
           lightRot[0] -= (timeDiff * velocity[2]);
           lightRot[0] += (timeDiff * velocity[3]);
                      
           if(lightRot[1]<0) lightRot[1]=0;
           if(lightRot[1]>85) lightRot[1]=85; 
}

//--------------------------------------------------------------------------------------DISPLAY
void display()
{
     //perform camera moves
     glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	glTranslatef(0,0,-cam_zoom);
	glTranslatef(cam_tx,cam_ty,0);
	glRotatef(cam_rotx,1,0,0);
	glRotatef(cam_roty,0,1,0);	
	
	// draw grid
	//----------ROB BATEMAN'S GRID CODE-------------------*****<ROBS CODE>******
	glDisable(GL_LIGHTING);
	glBegin(GL_LINES);
	for(int i=-10;i<=10;++i) {
		glVertex3f(i,0,-10);
		glVertex3f(i,0,10);

		glVertex3f(10,0,i);
		glVertex3f(-10,0,i);
	}
	glEnd();
	//---------------------------------------------------******</ROBS CODE>*****
	
	glEnable(GL_LIGHTING);
	glEnable(GL_COLOR_MATERIAL);
	//draw the light
	glColorMaterial(GL_FRONT, GL_EMISSION);
    glColor3f(1, 1, 1);
	glPushMatrix();
    glRotatef(lightRot[0],0,1,0);
   	glRotatef(lightRot[1],0,0,1);
    glTranslatef(30,0,0);
	glutSolidSphere(0.5, 10, 10);
	glPopMatrix();
	glColorMaterial(GL_FRONT, GL_EMISSION);
    glColor3f(0, 0, 0);
	
    //draw the plant
    glutSolidSphere(0.2, 10, 10);
    glColorMaterial(GL_FRONT, GL_DIFFUSE);
    glColor3f(0.2, 0.25, 0.2);
    glColorMaterial(GL_FRONT, GL_SPECULAR);
    glColor3f(0.1, 0.2, 0.1);
    stem_ptr->drawPlant();
    glColorMaterial(GL_FRONT, GL_DIFFUSE);
    glColor3f(0.7, 0.7, 0.7);
    glColorMaterial(GL_FRONT, GL_SPECULAR);
    glColor3f(0.1, 0.1, 0.1);
    root_ptr->drawPlant();
    glDisable(GL_COLOR_MATERIAL);
    SDL_GL_SwapBuffers();
}

//------------------------------------------------------------------------------------------ MIC INPUT

void micInput(float* growthRate, float* phototropismRate){
     
     // Update FMOD    
        FMOD_System_Update(fmodSystem);   
        
       	float pitch, volume, a;
       	Uint32 timeNow = SDL_GetTicks();
        
        // Variables for calculating volume
        float wave[512];
        float maxVol = 0;

        // Variables for calculating pitch
        float spectrum[SPECTRUMSIZE];
        float maxFreq = 0;
        float dominanthz = 0;
        float bin = 0;      
        
        // Get wave data
        result = FMOD_Channel_GetWaveData(channel,wave,512,0);
        ERRCHECK(result);
        
        // Get max volume of current sample
        for(int i=0;i!=512;i++)
        {                        
            if(wave[i]>maxVol)
            {
                maxVol = wave[i];
            }
        }
        
        // Get spectum data
        result = FMOD_Channel_GetSpectrum(channel,spectrum,SPECTRUMSIZE,0,FMOD_DSP_FFT_WINDOW_TRIANGLE);
        ERRCHECK(result);           

        // Get max frequency of sample
        for (int i= 0;i< 8191;i++)
        {
            if (spectrum[i] > 0.01f && spectrum[i] > maxFreq)
            {
                maxFreq = spectrum[i];
                bin = (float)i;
            }
        }
        
        // Calculate dominant frequency
        dominanthz  = bin * BINSIZE;
        volumeAverage+=maxVol;
        frequencyAverage+=dominanthz;
        numValues+=1;
        
        // Update numSeconds - numSeconds shows how many seconds have
        // passed since the program was launched.
        if(timeNow>=(numSeconds+1000))
        {
            numSeconds+=1000;
        }
        
        // Update maxVol values every 0.5 seconds
        if(timeNow>=(updateTimer+125))
        {
            volumeAverage = (volumeAverage/numValues);
            frequencyAverage = (frequencyAverage/numValues);             
                                                        
            maxVolThen = maxVolNow;
            maxVolNow = volumeAverage;
            freqThen = freqNow;
            freqNow = frequencyAverage;
            
            volumeAverage = 0.0;
            frequencyAverage = 0.0;
            numValues = 0;           
            updateTimer = (updateTimer+125);
        }
	
    	// Calculate the interpolation coefficient 'a'
    	a = float(timeNow-numSeconds);
    	a = a/1000;
    	// Ranging from 0 to 1 every 0.5 seconds
    	a = (a - 0.125) * 8;
    	// If 'a' is negative make it positive
    	if(a<0)
    	{
               a = 1 + a;
        }
        
        a = fmod( a, 1 );
        
    	// Calculate the linear interpolation	
    	volume = maxVolThen + (a*(maxVolNow - maxVolThen));
    	pitch = freqThen + (a*(freqNow - freqThen));	
    	//*growthRate = 0.0002 * volume;
        //*phototropismRate = (0.002 * (pitch /500));
        
     
     }

//------------------------------------------------------------------------------------------ NEW PLANT

void newPlant(void){
     // free up all the memory for the old plant
      while( !plantMemory.empty() ){
             delete [] (branch*)plantMemory.top();
             plantMemory.pop();
             }
             
    //--------------------------- make the stem ---------------------
    branch *aBranch;
    aBranch = new branch [10];
    plantMemory.push(aBranch);
    branch *lastBranch_ptr = NULL;
    stem_ptr = &aBranch[0];
    
    for(int i=0; i < 10; i++){
            
            float initialLength = 0.0;
            float finalLength = 2;
            float bendMax = 10;
            float rotX = (-10+rand()%20);
            float rotY = 0;
            float rotZ = (-10+rand()%20);
            float radius = 0.2;
            int splits = 0;
            if ( (i+1)%2 == 0 ) splits = 2;
            
            //---------initialize the node
            aBranch[i] = branch(false, 1, radius, initialLength, finalLength, lastBranch_ptr, rotX, rotY, rotZ, bendMax, splits);
            //---------set its parent to point to it
            if (aBranch[i].getParent() != NULL){
                               branch *parentBranch_ptr = aBranch[i].getParent();
                               parentBranch_ptr->setChild(&aBranch[i]);
                               } 
            //---------set the last branch to be this one                          
            lastBranch_ptr = &aBranch[i];
            
            }
            
        //--------------------------- make the root ---------------------
        branch *bBranch;
        bBranch = new branch [10];
        plantMemory.push(bBranch);
        lastBranch_ptr = NULL;
        root_ptr = &bBranch[0];
        
        for(int i=0; i < 10; i++){
                
                float initialLength = 0.0;
                float finalLength = 2;
                float bendMax = 5;
                float rotX = (-10+rand()%20);
                float rotY = 0;
                float rotZ = (-10+rand()%20);
                float radius = 0.1;
                int splits = 0;
                if ( (i+1)%2 == 0 ) splits = 2;
                if (i == 0 ) splits = 4;
                
                //---------initialize the node
                bBranch[i] = branch(true, 1, radius, initialLength, finalLength, lastBranch_ptr, rotX, rotY, rotZ, bendMax, splits);
                //---------set its parent to point to it
                if (bBranch[i].getParent() != NULL){
                                   branch *parentBranch_ptr = bBranch[i].getParent();
                                   parentBranch_ptr->setChild(&bBranch[i]);
                                   } 
                //---------set the last branch to be this one                          
                lastBranch_ptr = &bBranch[i];
                
                }
          
     
     
     }

//------------------------------------------------------------------------------------------ MAIN


int main(int argc, char *argv[])
{
    //---------------------------- Initialise FMOD -----------------------------
	
	// Create the main system object
    result = FMOD_System_Create(&fmodSystem);
    ERRCHECK(result);
     
    // Initialise FMOD to default settings
	result = FMOD_System_Init(fmodSystem, 100, FMOD_INIT_NORMAL, 0);
	ERRCHECK(result);

	memset(&exinfo, 0, sizeof(FMOD_CREATESOUNDEXINFO));
	
	exinfo.cbsize           = sizeof(FMOD_CREATESOUNDEXINFO);
	exinfo.numchannels      = 1;
	exinfo.format           = FMOD_SOUND_FORMAT_PCM16;
	exinfo.defaultfrequency = OUTPUTRATE;
	exinfo.length           = 11025;

	result = FMOD_System_CreateSound(fmodSystem, 0, FMOD_2D | FMOD_SOFTWARE | FMOD_LOOP_NORMAL | FMOD_OPENUSER, &exinfo, &sound);
	ERRCHECK(result);

	result = FMOD_System_RecordStart(fmodSystem, sound, true);
	ERRCHECK(result);

	result = FMOD_System_PlaySound(fmodSystem, FMOD_CHANNEL_REUSE, sound, false, &channel);
	ERRCHECK(result);
	
	result = FMOD_Channel_SetVolume(channel, 0.0);
	ERRCHECK(result);
	
	//------------------- Initialize SDL --------------------------------------
  SDL_Event event;
  SDL_Init(SDL_INIT_VIDEO);
  SDL_WM_SetCaption  ("lightGrow", "lightGrow.ico");
  SDL_SetVideoMode(800, 600, 32, SDL_OPENGL | SDL_HWSURFACE | SDL_RESIZABLE);
/*  
 GLenum err = glewInit();
if (GLEW_OK != err)
{
  // Problem: glewInit failed, something is seriously wrong. 
  fprintf(stderr, "Error: %s\n", glewGetErrorString(err));
}
fprintf(stdout, "Status: Using GLEW %s\n", glewGetString(GLEW_VERSION));
*/

Uint32 timeNow;
Uint32 timeThen = 0;

srand ( time(NULL) );

  glViewport(0, 0, 800, 600);
  glClearColor(0.3f, 0.3f, 0.4f, 0.0f);
  glClearDepth(1.0);
  glDepthFunc(GL_LESS);
//-----------------------setup the materials and shading
	GLfloat mat_specular[] = {0.6, 0.6, 0.6, 0.3};
	GLfloat mat_shininess[] = {100.0};
	GLfloat light_position[] = {10, 10, 10, 0.0};
	GLfloat white_light[] = {1.0, 1.0, 1.0, 1.0};
	GLfloat lmodel_ambient[] = {0.2, 0.2, 0.2, 1.0};

	glShadeModel(GL_SMOOTH);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, mat_shininess);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, white_light);
	glLightfv(GL_LIGHT0, GL_SPECULAR, white_light);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);

//-----------------------enables
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_DEPTH_TEST);
	
  glMatrixMode(GL_PROJECTION);
  gluPerspective(45,(float)800/600,0.1,200);
  glMatrixMode(GL_MODELVIEW);
 
//-----------------------growth control variables
  float growthRate = 0.0001;   //units per millisecond
  float phototropismRate = 0.0005; //degrees per millisecond

//-------------------------start an initial plant
  newPlant();


//---------------------------------------------------------------------------------------------------------
//----------------------------------------  MASTER LOOP  --------------------------------------------------
//---------------------------------------------------------------------------------------------------------
  bool pause = false;
  bool done = false;
  while(!done)
  {
              //printf("\n NEW FRAME \n\n");
           while(SDL_PollEvent(&event)) 
           {
           switch(event.type){
                              case SDL_MOUSEBUTTONDOWN:
                                   if(event.button.button == SDL_BUTTON_LEFT){
                                                   mouseButtons[0] = 1;
                                                   }
                                   else if(event.button.button == SDL_BUTTON_MIDDLE){
                                                   mouseButtons[1] = 1;
                                                   }
                                   else if(event.button.button == SDL_BUTTON_RIGHT){
                                                   mouseButtons[2] = 1;
                                                   }                                                   
                                                   cam_lastx = event.button.x;
                                                   cam_lasty = event.button.y;
                                                   break;
                                                   
                              case SDL_MOUSEBUTTONUP:
                                   if(event.button.button == SDL_BUTTON_LEFT){
                                                   mouseButtons[0] = 0;
                                                   }
                                   else if(event.button.button == SDL_BUTTON_MIDDLE){
                                                   mouseButtons[1] = 0;
                                                   }
                                   else if(event.button.button == SDL_BUTTON_RIGHT){
                                                   mouseButtons[2] = 0;
                                                   }
                                                   break;
                              case SDL_MOUSEMOTION:
                                   motion(event.motion.x, event.motion.y);
                                   break;
                                   
                              case SDL_KEYDOWN:
                                   if(event.key.keysym.sym == SDLK_ESCAPE){
                                                           done = true;
                                                           }
                                   if(event.key.keysym.sym == SDLK_UP){
                                                   upPressed = true;
                                                   }
                                   if(event.key.keysym.sym == SDLK_DOWN){
                                                   downPressed = true;
                                                   }
                                   else if(event.key.keysym.sym == SDLK_LEFT){
                                                   leftPressed = true;
                                                   }
                                   else if(event.key.keysym.sym == SDLK_RIGHT){
                                                   rightPressed = true;
                                                   }
                                                   
                                   if(event.key.keysym.sym == SDLK_p){
                                                           if(pause){
                                                                     pause = false;
                                                                     growthRate = 0.0001;phototropismRate = 0.0005;
                                                                     }
                                                           else pause = true;
                                                           }
                                   if(event.key.keysym.sym == SDLK_RETURN){
                                                           newPlant();
                                                           }
                                                           
                                                   break;
                                                   
                              case SDL_KEYUP:
                                   if(event.key.keysym.sym == SDLK_UP){
                                                   upPressed = false;
                                                   }
                                   if(event.key.keysym.sym == SDLK_DOWN){
                                                   downPressed = false;
                                                   }
                                   else if(event.key.keysym.sym == SDLK_LEFT){
                                                   leftPressed = false;
                                                   }
                                   else if(event.key.keysym.sym == SDLK_RIGHT){
                                                   rightPressed = false;
                                                   }
                                                   break; 
                                                   
                              case SDL_VIDEORESIZE:
                                   SDL_SetVideoMode(event.resize.w, event.resize.h, 32, SDL_OPENGL | SDL_HWSURFACE | SDL_RESIZABLE);
                                   resize(event.resize.w, event.resize.h);                                   
                                   break;                  
                                                   
                              case SDL_QUIT:
                                   done = true;
                                   break;
                                   
                                   }
           }
           
  //------------------calculate the microphone input
  micInput(&growthRate, &phototropismRate);
  
  //-----------------if pause is activated stop any growth
  if(pause){
            growthRate = 0;   
            phototropismRate = 0; 
            }
          
  //------------------calculate the time diffrence to last frame        
  timeNow = SDL_GetTicks();
  float timeDiff = timeNow - timeThen;
  timeThen = timeNow;
  
  //------------------update light position       
  moveLight(timeDiff);
  glLoadIdentity(); 
  glRotatef(lightRot[0],0,1,0);
  glRotatef(lightRot[1],0,0,1);
  glTranslatef(30,0,0);
  float lightMatrix[16];
    glGetFloatv(GL_TRANSPOSE_MODELVIEW_MATRIX, lightMatrix);
    Mat4 lightWorldSpaceBase(lightMatrix[0], lightMatrix[1], lightMatrix[2], lightMatrix[3],
                         lightMatrix[4], lightMatrix[5], lightMatrix[6], lightMatrix[7],
                          lightMatrix[8], lightMatrix[9], lightMatrix[10], lightMatrix[11],
                           lightMatrix[12], lightMatrix[13], lightMatrix[14], lightMatrix[15]);
                           
    Vec4 ident(0.0, 0.0, 0.0, 1.0);                       
    Vec4 lightPos = lightWorldSpaceBase * ident;
    
    float light_position2[4] = {lightPos[0], lightPos[1], lightPos[2], 1.0};
    glLightfv(GL_LIGHT0, GL_POSITION, light_position2);
  
           
  //------------------navigate the branch nodes and perform the growth
  
  float growth = growthRate * timeDiff;
  float bend = phototropismRate * timeDiff;
  //grow the stems
  glLoadIdentity();
  stem_ptr->phototropism(lightPos, bend, growth);
  //grow the roots
  Vec4 gravityVector(0,-200,0,1);
  glLoadIdentity();
  glRotatef(180,0,0,1);
  root_ptr->phototropism(gravityVector, bend, growth);

           
  //------------------navigate the branch nodes and update the vertex rings
  //update the stems
  glLoadIdentity();
  stem_ptr->updateVertexRing();
  //update the roots
  glLoadIdentity();
  glRotatef(180,0,0,1);
  root_ptr->updateVertexRing();
  
        
    //-----------------display a frame now!-------------------------   
    display();
    
  }
//---------------------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------------------

 // Exit FMOD
 result = FMOD_Sound_Release(sound);
 ERRCHECK(result);
 result = FMOD_System_Close(fmodSystem);
 ERRCHECK(result);
 result = FMOD_System_Release(fmodSystem);
 ERRCHECK(result);
 // Exit SDL
 SDL_Quit();
 
  return(0);
}
